#include <iostream>
#include <string>

#include "TStyle.h"
#include "TPad.h"
#include "TROOT.h"
#include "TSystem.h"
#include "TList.h"
#include "TDirectory.h"
#include "TObject.h"
#include "TObjectTable.h"
#include "TObjArray.h"
#include "TApplication.h"
#include "TCanvas.h"
#include "TLine.h"
#include "TFrame.h"
#include "TGraph.h"
#include "TGraphErrors.h"
#include "TH2.h"
#include "TAxis3D.h"
#include "TF1.h"
#include "TView.h"
#include "TString.h"
#include "TPolyMarker3D.h"
#include "TPolyLine3D.h"
#include "TMinuit.h"
#include "TLatex.h"
#include "TLine.h"

#ifdef WIN32
#define CROOT_DLL_EXPORTS
#define WIN32_LEAN_AND_MEAN		// Exclude rarely-used stuff from Windows headers
#include <windows.h>
#endif
#include "croot.h"

//------------------------------------------------------
// My Classes def
class TPolyL3D : public TPolyLine3D{
private:
  TString fName;
public:
  TPolyL3D(int n);
  const char *GetName() const {return fName.Data();};
  void SetName(const char *name){fName=name;};
};

class TPolyM3D : public TPolyMarker3D{
private:
  TString fName;
public:
  TPolyM3D(int n);
  const char *GetName() const {return fName.Data();};
  void SetName(const char *name){fName=name;};
};

class TGraphErrs : public TGraphErrors{
private:
  int lup;
public:
  TGraphErrs(int n, double *x, double *y, double *ex, double *ey);
  const int GetUp() const {return lup;};
  void SetUp(int i){lup=i;};
};

// My Classes code
TPolyL3D::TPolyL3D(int n):TPolyLine3D(n){}
TPolyM3D::TPolyM3D(int n):TPolyMarker3D(n){}
TGraphErrs::TGraphErrs(int n, double *x, double *y, double *ex, double *ey):
  TGraphErrors(n,x,y,ex,ey){}

//------------------------------------------------------


TObjArray    ROOT_pl;

float        ROOT_loffx=0.0,ROOT_loffy=0.0,ROOT_loffz=0.0;
TCanvas      *ROOT_canv;
int          ROOT_lhandle=0;
int          ROOT_lupdate=1;
short int    ROOT_lcolor =0;
short int    ROOT_lwait  =1;
short int    ROOT_lgrid  =0;
int          ROOT_def_nx=1,ROOT_def_ny=1;
int          ROOT_npad=1;

int          ROOT_lfirst=0;

TApplication *ROOT_theApp=NULL;


//----------------------------------------
// Useful functions.. Not for the user
//----------------------------------------

#ifdef WIN32
BOOL __stdcall DllMain( HANDLE hModule, 
                        DWORD  ul_reason_for_call, 
                        LPVOID lpReserved
					 )
{
    switch (ul_reason_for_call)
	{
		case DLL_PROCESS_ATTACH:
		case DLL_THREAD_ATTACH:
		case DLL_THREAD_DETACH:
		case DLL_PROCESS_DETACH:
			break;
    }
    return TRUE;
}
#endif

TObject* Check_His(const char *hist, const char *routine){
  
  TObject* tmp = gDirectory->FindObject(hist);
  if (tmp == NULL){
    std::cout << "%CRoot error in " <<routine <<": histogram " 
    << hist << " not existing" << std::endl;
  }
  return tmp;

}

TObject* Find_ObjectInPad(const char *type){
  
  TIter next(gPad->GetListOfPrimitives());
  TObject *obj;
  TObject *found=NULL;

  while ((obj = next())){
    if (obj->InheritsFrom(type)){
      found = obj;
    }
  }
  return found;
}


std::string GetStringName(std::string name,int i){

  std::string str;
  std::string vchar[10]={"0","1","2","3","4","5","6","7","8","9"};

  if (i<100) {
    if (i<10) 
      str = name + vchar[i%10] + '\0';
    else
      str = name + vchar[i/10] + vchar[i%10] + '\0';
  } else {
    std::cout << 
      "%CROOT: error - Too many strings:" << i << 
    std::endl;
    exit(1);
  }
  return str;

}


void Handle_Wait(int loc_lwait, int imode){

  char c;

  if (loc_lwait == 2) {
    if (imode == 0) {
      std::cout << "%CROOT message: select Quit ROOT from File menu to change page" << std::endl;      
    } else {
      std::cout << "%CROOT message: select Quit ROOT from File menu to.. Quit ROOT" << std::endl;      
    }
    ROOT_theApp->Run(kTRUE);
  } else {
    if (imode == 0) {
      while (1){
        std::cout << "%CROOT message: enter n to change page" << std::endl;      
        std::cin >> c;
        if (c=='n' || c=='N') break;
      } 
    } else {
      while (1){
        std::cout << "%CROOT message: enter q to Quit ROOT" << std::endl;      
        std::cin >> c;
        if (c=='q' || c=='Q') break;
      }
    }
  }

}

void Handle_Pad(){

  if (ROOT_lhandle == 0){
    if (ROOT_npad          < ROOT_def_nx*ROOT_def_ny){
      ROOT_npad++;
      R_cd(ROOT_npad);
    } else {
      R_Divide(ROOT_def_nx,ROOT_def_ny);
    }
  }
  if (ROOT_lcolor != 0) gPad->SetFillColor(ROOT_lcolor);
  if (ROOT_lgrid  != 0) gPad->SetGrid();
  gPad->SetLogy(gStyle->GetOptLogy());
  gPad->SetLogx(gStyle->GetOptLogx());
  gPad->SetLogz(gStyle->GetOptLogz());

  ROOT_lhandle = 0;

}

void R_HandleTitleOffset(float off, const char *axis){

  TAxis3D *ax =  (TAxis3D *) gPad->FindObject("axis3druler");
  TH1F    *hf =  (TH1F *) gPad->GetPrimitive("hframe");
  TH1F    *h1 =  (TH1F *) Find_ObjectInPad("TH1");
  TH2F    *h2 =  (TH2F *) Find_ObjectInPad("TH2");
  TGraph       *xgr  = (TGraph *) Find_ObjectInPad("TGraph");
  TGraphErrs *xgre = (TGraphErrs *) Find_ObjectInPad("TGraphErrors");

  if (ax != NULL){
    ax->SetTitleOffset(off,axis);
  } else if (hf != NULL){
    hf->SetTitleOffset(off,axis);
  } else if (h1 != NULL){
    h1->SetTitleOffset(off,axis);
  } else if (h2 != NULL){
    h2->SetTitleOffset(off,axis);
  } else if (xgr != NULL){
    xgr->GetHistogram()->SetTitleOffset(off,axis);
  } else if (xgre != NULL){
    xgre->GetHistogram()->SetTitleOffset(off,axis);
  } else {
    std::cout << "%CROOT error: no frame drawn" << std::endl;
  } 
}


void  R_Delete(const char *name){

  TObject  *objx=NULL;
  TObject  *pad;
  TObject  *del=NULL;
  int i,mpad=ROOT_def_nx*ROOT_def_ny;
  int *vpad=NULL;

  if (mpad != 1){

    vpad = new int[mpad];
    for (i=0;i<mpad;i++) vpad[i]=0;

    TIter next(ROOT_canv->GetListOfPrimitives());

    while ((pad = next())){
      if (pad->InheritsFrom("TPad")){
        objx = ((TPad *) pad)->FindObject(name);
        if (objx != NULL) {
          vpad[i]=1;
          del = objx;
        }
      }
    }

    if (del!=NULL) {
      ROOT_pl.Remove(del);
      delete del;
    }

    while ((pad = next())){
      if (vpad[((TPad *) pad)->GetNumber()]==1){
        ((TPad *) pad)->Modified();
      }
    }

  } else {

    objx = gPad->FindObject(name);
    if (objx!=NULL) delete objx;
    gPad->Modified();

  }
  ROOT_canv->Update();
  if (vpad!=NULL) delete [] vpad;
}

void Do_Modify(const char *name){

  TObject *objx=NULL; 
  TObject *pad;

  if (ROOT_canv != NULL){
    TIter next(ROOT_canv->GetListOfPrimitives());
    while ((pad = next())){
      if (pad->InheritsFrom("TPad")){
        objx = ((TPad *) pad)->FindObject(name);
        if (objx != NULL) {
          ((TPad *) pad)->Modified();
        }
      }
    }
  }

}


//-----------------------------------------------------------
//
// ** Settings **
//
//-----------------------------------------------------------

CROOT_API void R_SetOptStat(int iopt){
  gStyle->SetOptStat(iopt);
}

CROOT_API void R_SetOptLogx(short int iopt){
  gStyle->SetOptLogx(iopt);
}

CROOT_API void R_SetOptLogy(short int iopt){
  gStyle->SetOptLogy(iopt);
}

CROOT_API void R_SetOptLogz(short int iopt){
  gStyle->SetOptLogz(iopt);
}


CROOT_API void R_SetGrid(short int iopt){
  ROOT_lgrid = iopt;
}

CROOT_API void R_SetWait(short int iopt){ 
  ROOT_lwait  = iopt;
}

CROOT_API void R_SetUpdate(short int iopt){ 
  ROOT_lupdate  = iopt;
}

CROOT_API void R_SetMarkerStyle(short int iopt){
  gStyle->SetMarkerStyle(iopt);
}

CROOT_API void R_SetMarkerColor(short int iopt){
  gStyle->SetMarkerColor(iopt);
}

CROOT_API void R_SetMarkerSize(float fopt){
  gStyle->SetMarkerSize(fopt);
}

CROOT_API void R_SetLineStyle(short int iopt){
  gStyle->SetLineStyle(iopt);
}

CROOT_API void R_SetLineColor(short int iopt){
  gStyle->SetLineColor(iopt);
}

CROOT_API void R_SetLineWidth(short int iopt){
  gStyle->SetLineWidth(iopt);
}

CROOT_API void R_SetHistLineStyle(short int iopt){
  gStyle->SetHistLineStyle(iopt);
}

CROOT_API void R_SetHistLineColor(short int iopt){
  gStyle->SetHistLineColor(iopt);
}

CROOT_API void R_SetHistFillStyle(short int iopt){
  gStyle->SetHistFillStyle(iopt);
}

CROOT_API void R_SetHistFillColor(short int iopt){
  gStyle->SetHistFillColor(iopt);
}

CROOT_API void R_SetHistLineWidth(short int iopt){
  gStyle->SetHistLineWidth(iopt);
}

CROOT_API void R_SetFuncStyle(short int iopt){
  gStyle->SetFuncStyle(iopt);
}

CROOT_API void R_SetFuncColor(short int iopt){
  gStyle->SetFuncColor(iopt);
}

CROOT_API void R_SetFuncWidth(short int iopt){
  gStyle->SetFuncWidth(iopt);
}

CROOT_API void R_SetFillColor(short int iopt){
  ROOT_lcolor = iopt;
}

CROOT_API void R_SetTitleOffset(float off, const char *axis){
  TString Opt=axis;
  Opt.ToLower();
  if (Opt.Contains("x"))
    ROOT_loffx = off;
  else if (Opt.Contains("y"))
    ROOT_loffy = off;
  else if (Opt.Contains("z"))
    ROOT_loffz = off;
}



//---------------------------------------------------------------------------
//  
//  ** Initialization part **
//
//  R_Init
//  R_End
//  R_cd
//  R_Divide
//  R_Update
//  R_Interact
//


CROOT_API void R_Update(void){
 
  gPad->Modified();
  ROOT_canv->Update(); 

}

CROOT_API void R_Interact(void){
  std::cout << "%CROOT message: select Quit ROOT from File menu to continue.." << std::endl;      
  ROOT_theApp->Run(kTRUE);
}

CROOT_API void R_Init(void){

  ROOT_lfirst = 0;
  if (ROOT_theApp==NULL)
    ROOT_theApp = new TApplication("App", 0, NULL);

  //  std::cout << theApp << " " << gROOT << std::endl;
  gROOT->Reset();
  R_SetWait(1);
  //  gStyle->ls();
  //  gObjectTable->Print();
  R_SetOptStat(0);
  R_SetMarkerStyle(20);
  R_SetFuncWidth(1);
  ROOT_canv = new TCanvas("plot","Root Window",200,10,700,500);
  ROOT_canv->SetHighLightColor(10);

}

CROOT_API void R_End(void){

  //  gObjectTable->Print();
  //  gDirectory->ls();
  if ( ROOT_lwait>0 ) Handle_Wait(ROOT_lwait,1);
  delete ROOT_canv;
  //delete theApp;

}


CROOT_API void R_cd(int nn){
 
  ROOT_npad    = nn;
  ROOT_lhandle = 1;
  ROOT_canv->cd(ROOT_npad);
  
}


CROOT_API void R_Divide(int nx, int ny){
  
  ROOT_npad    = 1;
  ROOT_def_nx  = nx;
  ROOT_def_ny  = ny;

  if ( ROOT_lfirst==1 && ROOT_lwait>0 ) Handle_Wait(ROOT_lwait,0);
  ROOT_lfirst = 1;
  
  gROOT->Reset();
  ROOT_canv->Clear();

  if (nx != 1 || ny !=1 ){
    ROOT_canv->Divide(nx,ny);
  }

  R_cd(ROOT_npad);

}


//---------------------------------------------------------------
//
// ** Basics graphical routines.. **
//
// R_DrawFrame
// R_ResizeFrame
// R_Graph
// R_GraphObj
// R_GraphErrors
// R_GraphErrorsObj
// R_PlotFunc1D       ?
// R_PlotFuncPar1D    ?
// R_Draw3DFrame
// R_3DGraph
// R_3DGraphObj
//
//---------------------------------------------------------------


CROOT_API void R_DrawFrame(double xmin, double xmax, double ymin, double ymax){

  Handle_Pad();
  gPad->DrawFrame(xmin,ymin,xmax,ymax);
  gPad->Modified();
  if (ROOT_lupdate==1) R_Update();;

}


CROOT_API void R_ResizeFrame(double xmin, double xmax, double ymin, double ymax){

  TH1F *obj = (TH1F *) gPad->FindObject("hframe");
  if (obj != NULL) {
    obj->GetXaxis()->SetLimits(xmin,xmax);
    obj->SetMinimum(ymin);
    obj->SetMaximum(ymax);
  }

  gPad->Modified();
  if (ROOT_lupdate==1) R_Update();;

}

CROOT_API void R_Graph(int n, double *x, double *y, const char *opt){

  TGraph *gr;
  TString Opt=opt;
  
  Opt.ToLower();
  if (Opt.Contains("a") && !Opt.Contains("same"))
    Handle_Pad();

  gr = new TGraph(n,x,y);
  gr->SetBit(kCanDelete);
  gr->Draw(opt);

  gPad->Modified();
  if (ROOT_lupdate==1) R_Update();;

}

CROOT_API void R_GraphObj(const char *name, int n, double *x, double *y, const char *opt){

  TGraph  *gr;
  TString Opt=opt;
  
  Opt.ToLower();
  if (Opt.Contains("a") && !Opt.Contains("same"))
    Handle_Pad();

  TObject *objgr = ROOT_pl.FindObject(name);
  if (objgr!=NULL){
    gr = (TGraph *)objgr;
    if (gr->GetN()!=n) gr->Set(n);
    for (int i=0;i<n;i++){
      gr->SetPoint(i,x[i],y[i]);
    }
  } else {
    gr = new TGraph(n,x,y);
    gr->SetBit(0);
    gr->SetNameTitle(name,"");
    ROOT_pl.Add(gr);
    gr->SetMarkerStyle(gStyle->GetMarkerStyle());
    gr->SetMarkerColor(gStyle->GetMarkerColor());
    gr->SetMarkerSize(gStyle->GetMarkerSize());
    gr->SetLineStyle(gStyle->GetLineStyle());
    gr->SetLineColor(gStyle->GetLineColor());
    gr->SetLineWidth(gStyle->GetLineWidth());
  }

  Opt.ToLower();
  if (Opt.Contains("m") && !Opt.Contains("same")){
    if (ROOT_def_nx != 1 && ROOT_def_ny != 1){
      Do_Modify(name);   
    } else {
      gPad->Modified();
    }
  } else {
    gr->Draw(opt); 
    gPad->Modified();
  }

  if (ROOT_lupdate==1) R_Update();;

}


CROOT_API void R_GraphErrors(int n, double *x, double *y, double *ex, double *ey, const char *opt){

  TGraphErrs  *gr;
  TString Opt=opt;

  Opt.ToLower();  
  if (Opt.Contains("a") && !Opt.Contains("same"))
    Handle_Pad();

  gr = new TGraphErrs(n,x,y,ex,ey);
  gr->SetBit(kCanDelete);
  gr->Draw(opt);

  gPad->Modified();
  if (ROOT_lupdate==1) R_Update();;

}

CROOT_API void R_GraphErrorsObj(const char *name, int n, double *x, double *y, double *ex, double *ey, const char *opt){

  TGraphErrs *gr;
  TString      Opt=opt;
  double       xmin,xmax,ymin,ymax;

  Opt.ToLower();  
  if (Opt.Contains("a") && !Opt.Contains("same"))
    Handle_Pad();

  TObject *objgr = ROOT_pl.FindObject(name);

  if (objgr!=NULL){
    gr = (TGraphErrs *)objgr;

    xmin=x[0]-ex[0];
    ymin=y[0]-ey[0];
    xmax=x[0]+ex[0];
    ymax=y[0]+ey[0];

    if (gr->GetN()!=n) gr->Set(n);
    for (int i=0;i<n;i++){
      gr->SetPoint(i,x[i],y[i]);
      gr->SetPointError(i,ex[i],ey[i]);
      if (gr->GetUp() == 1){
        if (xmin > x[i]-ex[i]) xmin = x[i]-ex[i];      
        if (ymin > y[i]-ey[i]) ymin = y[i]-ey[i];      
        if (xmax < x[i]+ex[i]) xmax = x[i]+ex[i];      
        if (ymax < y[i]+ey[i]) ymax = y[i]+ey[i];      
      }
    }

    if (gr->GetUp() == 1 ){
      R_SetAxis(name,xmin,xmax,ymin,ymax);
    }

  } else {
    gr = new TGraphErrs(n,x,y,ex,ey);
    gr->SetNameTitle(name,"");
    gr->SetBit(0);
    gr->SetUp(0);
    ROOT_pl.Add(gr);
    gr->SetMarkerStyle(gStyle->GetMarkerStyle());
    gr->SetMarkerColor(gStyle->GetMarkerColor());
    gr->SetMarkerSize(gStyle->GetMarkerSize());
    gr->SetLineStyle(gStyle->GetLineStyle());
    gr->SetLineColor(gStyle->GetLineColor());
    gr->SetLineWidth(gStyle->GetLineWidth());

  }

  if (Opt.Contains("m") && !Opt.Contains("same")){
    if (ROOT_def_nx != 1 && ROOT_def_ny != 1){
      Do_Modify(name);   
    } else {
      gPad->Modified();
    }
  } else {
    if (Opt.Contains("a") && !Opt.Contains("same"))
      gr->SetUp(1);
    gr->Draw(opt); 
    gPad->Modified();
  }
  
  if (ROOT_lupdate==1) R_Update();;

}


CROOT_API void R_SetAxis(const char *name,double xmin, double xmax, double ymin, double ymax){

  TH1F   *th1f=NULL;

  TObject *objgr = ROOT_pl.FindObject(name);
  if (objgr!=NULL){
    if (objgr->InheritsFrom("TGraphErrors"))
      th1f = (TH1F *) ((TGraphErrs *) objgr)->GetHistogram();       
    else if (objgr->InheritsFrom("TGraph")) 
      th1f = (TH1F *) ((TGraph *) objgr)->GetHistogram();       
    th1f->GetXaxis()->SetLimits(xmin,xmax);
    th1f->SetMinimum(ymin);
    th1f->SetMaximum(ymax);
  }
  if (ROOT_def_nx != 1 && ROOT_def_ny != 1){
    Do_Modify(name);   
  } else {
    gPad->Modified();
  }
  if (ROOT_lupdate==1) R_Update();;

}

CROOT_API void R_SetRangeHist(const char *hist,int imin, int imax, const char *axis){

  TString Opt=axis;
  Opt.ToLower();

  TObject *objgr = ROOT_pl.FindObject(hist);
  if (objgr!=NULL){
    if (objgr->InheritsFrom("TH2")){
      if (Opt.Contains("x")) 
        ((TH2F *) objgr)->GetXaxis()->SetRange(imin,imax);
      else if (Opt.Contains("y")) 
        ((TH2F *) objgr)->GetYaxis()->SetRange(imin,imax);
    } else if (objgr->InheritsFrom("TH1")){
      ((TH1F *) objgr)->GetXaxis()->SetRange(imin,imax);
    }
  } 

  if (ROOT_def_nx != 1 && ROOT_def_ny != 1){
    Do_Modify(hist);   
  } else {
    gPad->Modified();
  }
  if (ROOT_lupdate==1) R_Update();;

}

CROOT_API void R_PlotFunc1D(double fun(double), double xmin, double xmax, const char *opt){

  int i;
  double x[1001],y[1001];
  TString Opt=opt;

  Opt.ToLower();  
  if (Opt.Contains("a") && !Opt.Contains("same"))
    Handle_Pad();

  if (gStyle->GetOptLogx()==0){
    for (i=0;i<1001;i++){
      x[i] = xmin+i*(xmax-xmin)/1000;
      y[i] = fun(x[i]);
    }
  } else {
    if (xmin<=0) {
      std::cout << "%CROOT error: R_PlotFunc1D xmin <= 0 !" << std::endl; 
      return;
    }
    for (i=0;i<1001;i++){
      x[i] = xmin*pow(10,i/1000.*(log10(xmax)-log10(xmin)));
      y[i] = fun(x[i]);
    }
  }

  TGraph *gr = new TGraph(1001,x,y);
  gr->SetBit(kCanDelete);
  if (Opt=="a") Opt+="c";
  if (Opt=="" || Opt==" "){
    gr->Draw("C");
  } else {
    gr->Draw(Opt);
  }

  gPad->Modified();
  if (ROOT_lupdate==1) R_Update();

}

CROOT_API void R_PlotFuncPar1D(double fun(double, double *), int npar, double *par, double xmin, double xmax, const char *opt){

  int i;
  double x[1001],y[1001];
  TString Opt=opt;

  Opt.ToLower();  
  if (Opt.Contains("a") && !Opt.Contains("same"))
    Handle_Pad();

  if (gStyle->GetOptLogx()==0){
    for (i=0;i<1001;i++){
      x[i] = xmin+i*(xmax-xmin)/1000;
      y[i] = fun(x[i],par);
    }
  } else {
    if (xmin<=0) {
      std::cout << "%CROOT error: R_PlotFuncPar1D xmin <= 0 !" << std::endl; 
      return;
    }
    for (i=0;i<1001;i++){
      x[i] = xmin*pow(10,i/1000.*(log10(xmax)-log10(xmin)));
      y[i] = fun(x[i],par);
    }
  }

  TGraph *gr = new TGraph(1001,x,y);
  gr->SetBit(kCanDelete);
  if (Opt=="a") Opt+="c";
  if (Opt=="" || Opt==" "){
    gr->Draw("C");
  } else {
    gr->Draw(Opt);
  }

  gPad->Modified();
  if (ROOT_lupdate==1) R_Update();

}

CROOT_API void R_Draw3DFrame(double xmin, double xmax, double ymin, double ymax, double zmin, double zmax, const char *opt){

  TString Opt=opt;

  Handle_Pad();
  
  Opt.ToLower();
  TView *view = new TView(1);
  view->SetBit(kCanDelete);
  view->SetRange(xmin,ymin,zmin,xmax,ymax,zmax);
  if (!Opt.Contains("n")) {
    view->ShowAxis();
  }
  gPad->Modified();
  if (ROOT_lupdate==1) R_Update();

}

CROOT_API void R_3DGraph(int n, double *x, double *y, double *z, const char *opt){

  TString Opt = opt;
  Opt.ToLower(); 

  if (Opt.Contains("l") || Opt.Contains("c")){

    TPolyLine3D   *l = new TPolyLine3D(n);
    l->SetBit(kCanDelete);

    for (int i=0;i<n;i++){
      l->SetPoint(i,x[i],y[i],z[i]); 
    }

    l->SetLineStyle(gStyle->GetLineStyle());
    l->SetLineColor(gStyle->GetLineColor());
    l->SetLineWidth(gStyle->GetLineWidth());
    l->Draw(); 

  } else {

    TPolyMarker3D *p = new TPolyMarker3D(n); 
    p->SetBit(kCanDelete);

    for (int i=0;i<n;i++){
      p->SetPoint(i,x[i],y[i],z[i]); 
    }

    p->SetMarkerStyle(gStyle->GetMarkerStyle());
    p->SetMarkerColor(gStyle->GetMarkerColor());
    p->SetMarkerSize(gStyle->GetMarkerSize());
    p->Draw(opt);  

  }

  gPad->Modified();
  if (ROOT_lupdate==1) R_Update();;

}


CROOT_API void R_3DGraphObj(const char *name,int n, double *x, double *y, double *z, const char *opt){

  int typ=0;

  TPolyL3D *l=NULL;
  TPolyM3D *p=NULL;

  TString Opt = opt;
  Opt.ToLower(); 

  TObject *objgr = ROOT_pl.FindObject(name);

  if (objgr == NULL){
    if (Opt.Contains("l") || Opt.Contains("c")){
      l = new TPolyL3D(n);
      for (int i=0;i<n;i++){
        l->SetPoint(i,x[i],y[i],z[i]);
      }
      l->SetLineStyle(gStyle->GetLineStyle());
      l->SetLineColor(gStyle->GetLineColor());
      l->SetLineWidth(gStyle->GetLineWidth());
      l->SetName(name); 
      typ = 0; ROOT_pl.Add(l);
    } else {
      p = new TPolyM3D(n);
      for (int i=0;i<n;i++){
        p->SetPoint(i,x[i],y[i],z[i]);
      }
      p->SetMarkerStyle(gStyle->GetMarkerStyle());
      p->SetMarkerColor(gStyle->GetMarkerColor());
      p->SetMarkerSize(gStyle->GetMarkerSize());
      p->SetName(name); 
      typ = 1; ROOT_pl.Add(p);
    }
  } else {
    if (objgr->InheritsFrom("TPolyLine3D")){
      l = (TPolyL3D *)objgr;
      for (int i=0;i<n;i++){
        l->SetPoint(i,x[i],y[i],z[i]); 
      }
      typ = 0;
    } else if (objgr->InheritsFrom("TPolyMarker3D")){
      p = (TPolyM3D *)objgr;
      for (int i=0;i<n;i++){
        p->SetPoint(i,x[i],y[i],z[i]); 
      }
      typ = 1;
    }
  }


  if (Opt.Contains("m")){
    if (ROOT_def_nx != 1 && ROOT_def_ny != 1)
      Do_Modify(name);
    else
      gPad->Modified();
  } else {
    if (typ==0){
      l->Draw(); 
    } else {
      p->Draw(); 
    }
    gPad->Modified();
  }
  if (ROOT_lupdate==1) R_Update();

}

//---------------------------------------------------------------------
//
// ** Histogram package **
//
//  R_CreateHist1D 
//  R_CreateHist2D
//  R_FillHist1D
//  R_FillHist2D
//  R_PlotHist
//  R_ResetHist
//  R_GetBinContentHist1D
//  R_GetBinContentHist2D
//
//---------------------------------------------------------------



CROOT_API void R_CreateHist1D(const char *hist, const char *title, int ncx, float lowx, float upx){

  TH1F* htemp;
  htemp = new TH1F(hist,title,ncx,lowx,upx);
  ROOT_pl.Add(htemp);

}


CROOT_API void R_CreateHist2D(const char *hist, const char *title, int ncx, float lowx, float upx, int ncy, 
                float lowy, float upy){

  TH2F* htemp;
  htemp = new TH2F(hist,title,ncx,lowx,upx,ncy,lowy,upy);
  ROOT_pl.Add(htemp);

}

CROOT_API void R_FillHist1D(const char *hist,float x, float w){

  TObject *obj = Check_His(hist,"R_FillHist1D");
  if (obj!=NULL){
    ((TH1F*) obj)->Fill(x,w);
  }
  
}

CROOT_API void R_FillHist2D(const char *hist,float x, float y, float w){

  TObject *obj = Check_His(hist,"R_FillHist2D");
  if (obj!=NULL){
    ((TH2F*) obj)->Fill(x,y,w);
  }
  
}

CROOT_API double R_GetBinContentHist1D(const char *hist, int bin){
 
  TObject *obj = Check_His(hist,"R_GetBinContentHist1D");
  if (obj != NULL){
    return ((TH1F *) obj)->GetBinContent(bin);
  } else {
    return -1;
  }

}

CROOT_API double R_GetBinContentHist2D(const char *hist, int binx, int biny){

  TObject *obj = Check_His(hist,"R_GetBinContentHist2D");
  if (obj != NULL){
    return ((TH2F *) obj)->GetBinContent(binx,biny);
  } else {
    return -1;
  }

}

CROOT_API int R_GetBinNumber(const char *hist, double x, const char *axis){

  TObject *obj;

  TString Opt=axis;
  Opt.ToLower();

  obj = Check_His(hist,"R_GetBinNumber");
  if (obj != NULL){
    if (obj->InheritsFrom("TH1"))
      return ((TH1F *) obj)->GetXaxis()->FindBin(x);
    else {
      if (Opt.Contains("x")) 
        return ((TH2F *) obj)->GetXaxis()->FindBin(x);
      else 
        return ((TH2F *) obj)->GetYaxis()->FindBin(x); 
    }
  } else {
    return -1; 
  }
}

CROOT_API double R_GetBinCenter(const char *hist, int ibin, const char *axis){

  TObject *obj;

  TString Opt=axis;
  Opt.ToLower();

  obj = Check_His(hist,"R_GetBinCenter");
  if (obj != NULL){
    if (obj->InheritsFrom("TH1"))
      return ((TH1F *) obj)->GetXaxis()->GetBinCenter(ibin);
    else {
      if (Opt.Contains("x")) 
        return ((TH2F *) obj)->GetYaxis()->GetBinCenter(ibin);
      else 
        return ((TH2F *) obj)->GetYaxis()->GetBinCenter(ibin);
    }
  } else {
    return -1;
  }
}


CROOT_API double R_GetEntries(const char *hist){

  TObject *obj;

  obj = Check_His(hist,"R_GetEntries");
  if (obj != NULL){
    if (obj->InheritsFrom("TH1")){
      return ((TH1F *) obj)->GetEntries();
    } else {
      return ((TH2F *) obj)->GetEntries();
    }
  } else {
    return -1;
  }

}

CROOT_API int R_GetNBins(const char *hist, const char *axis){

  TObject *obj;

  TString Opt=axis;
  Opt.ToLower();

  obj = Check_His(hist,"R_GetNBins");
  if (obj != NULL){
    if (obj->InheritsFrom("TH1")){
      return ((TH1F *) obj)->GetNbinsX();
    } else {
      if (Opt.Contains("x"))
        return ((TH2F *) obj)->GetNbinsX();
      else 
        return ((TH2F *) obj)->GetNbinsY();
    }
  } else {
    return -1;
  }

}

CROOT_API double R_GetRMS(const char *hist, int axis){

  TObject *obj;

  obj = Check_His(hist,"R_GetRms");
  if (obj != NULL){
    if (obj->InheritsFrom("TH1")){
      return ((TH1F *) obj)->GetRMS(axis);
    } else {
      return ((TH2F *) obj)->GetRMS(axis);
    }
  } else {
    return -1;
  }

}

CROOT_API double R_GetIntegral(const char *hist, const char *opt){

  TObject *obj;

  obj = Check_His(hist,"R_GetIntegral");
  if (obj != NULL){
    if (obj->InheritsFrom("TH1")){
      return ((TH1F *) obj)->Integral(opt);
    } else {
      return ((TH2F *) obj)->Integral(opt);
    }
  } else {
    return -1;
  }

}

CROOT_API double R_GetMean(const char *hist, int axis){

  TObject *obj;

  obj = Check_His(hist,"R_GetMean");
  if (obj != NULL){
    if (obj->InheritsFrom("TH1")){
      return ((TH1F *) obj)->GetMean(axis);
    } else {
      return ((TH2F *) obj)->GetMean(axis);
    }
  } else {
    return -1;
  }

}



CROOT_API void R_PlotHist(const char *hist,const char *opt){

  TObject *obj;

  TH1F *h1;
  TH2F *h2;

  TString Opt = opt;

  Opt.ToLower();
  if (!Opt.Contains("same") && !Opt.Contains("m")) 
    Handle_Pad();
        
  obj = Check_His(hist,"R_PlotH");

  if (obj != NULL){
    if (Opt.Contains("m") && !Opt.Contains("same")){
      if (ROOT_def_nx != 1 && ROOT_def_ny != 1){
        Do_Modify(hist);   
      } else {
        gPad->Modified();
      }
    } else {
      if (obj->InheritsFrom("TH1")){
        h1 = (TH1F *) obj;
        h1->Draw(opt);
        h1->SetLineColor(gStyle->GetHistLineColor());
        h1->SetLineStyle(gStyle->GetHistLineStyle());
        h1->SetLineWidth(gStyle->GetHistLineWidth());
        h1->SetFillStyle(gStyle->GetHistFillStyle());
        h1->SetFillColor(gStyle->GetHistFillColor());
      } else if (obj->InheritsFrom("TH2")){
        h2 = (TH2F *) obj;
        h2->Draw(opt);
        h2->SetLineColor(gStyle->GetHistLineColor());
        h2->SetLineStyle(gStyle->GetHistLineStyle());
        h2->SetLineWidth(gStyle->GetHistLineWidth());
        h2->SetFillStyle(gStyle->GetHistFillStyle());
        h2->SetFillColor(gStyle->GetHistFillColor());
      }
      if (ROOT_lgrid  == 1) gPad->SetGrid();
      if (ROOT_lcolor != 0) gPad->SetFillColor(ROOT_lcolor);
      gPad->Modified();
    }
  }

  if (ROOT_lupdate==1) R_Update();;

}

CROOT_API void R_ResetHist(const char *hist){

  TObject *obj;

  obj = Check_His(hist,"R_ResetH");
  if (obj != NULL){
    if (obj->InheritsFrom("TH1")){
      ((TH1F*) obj)->Reset();
    } else if (obj->InheritsFrom("TH2")){
      ((TH2F*) obj)->Reset();
    }
  }
  gPad->Modified();

}

//-------------------------------------------------------------------
//
// ** Minuit package **
//
// R_Minuit
// R_MinuitLimit
// R_MinuitSimpl
//
//-----------------------------------------------------------

static double (*xfun)(int np, double *xval);

void UserFun(int &npar, double *gin, double &f, double *x, int ifl){
  f =  (*xfun)(npar,x);
}

CROOT_API void R_Minuit(double (*fun)(int np, double *xval),  const char *mode, int npar, 
  char **cpar, double *par, double *step, double *min, double *max, 
  double *outpar, double *err,  double *chiq){

  char   theline[101];
  double arglist[100];
  int    ierflg,nvpar,nparx;
  double edm,errdef;
  int i;

  xfun = fun;
  TMinuit *gMinuit = new TMinuit(npar);
  gMinuit->SetBit(kCanDelete);

  // Set user function
  gMinuit->SetFCN(UserFun);
  // Set limits..
  for (i=0;i<npar;i++){
    TString Str = cpar[i];
    gMinuit->mnparm(i,Str,par[i],step[i],min[i],max[i],ierflg);
  }
  // Minuit initialization  
  arglist[0] = 1;
  gMinuit->mnexcm("CALL FCN" , arglist, 1, ierflg);
  arglist[0] = 0;
  gMinuit->mnexcm("SET PRINT", arglist, 1, ierflg);
  arglist[0] = 1;
  gMinuit->mnexcm("SET ERR"  , arglist, 1, ierflg);
  // Minimization
  TString Opt = mode;  
  TString Cmd;
  Opt.ToLower();
  if (Opt.Contains("m")){
    std::cout << "%CROOT message: Minuit: Entering interactive mode" << std::endl;
    do {  
      std::cout << "Minuit command > ";
      std::cin.getline(theline,100);
      ierflg = gMinuit->Command(theline);
      Cmd = theline;
      Cmd.ToLower();
    } while (!Cmd.Contains("exit"));
    std::cout << "%CROOT message: Minuit : End session" << std::endl;
  } else {
    gMinuit->mnexcm("MINI"     , arglist, 0, ierflg);
  }
  // Output of the parameter
  for (i=0;i<npar;i++){
    TString Str = cpar[i];
    gMinuit->mnpout(i,Str,outpar[i],err[i],min[i],max[i],ierflg);
  }
  // Output of the Chisquare
  gMinuit->mnstat(*chiq,edm,errdef,nvpar,nparx,ierflg);

}



CROOT_API void R_MinuitSimpl(double (*fun)(int np, double *xval), int npar, double *par, 
    double *outpar, double *err,  double *chiq){

    int    i;
    double *step,*min,*max;
    
    

    // Assigning names...
        // Assigning names...
    char **names = new char*[npar];
    for (i=0;i<npar;i++) {
      std::string str = GetStringName("p",i);
      names[i] = new char[str.length()+1];
      strncpy(names[i], str.c_str(), str.length());
    }


    // Assigning bounds..

    step = new double[npar];
    min  = new double[npar];
    max  = new double[npar];
    for (i=0;i<npar;i++){
      step[i] = 0.0001;
      min[i] = max[i] = 0.0;
    }

    R_Minuit(fun," ",npar,names,par,step,min,max,outpar,err,chiq);

    delete[] step;
    delete[] min;
    delete[] max;
    for (i=0; i<npar; i++) delete[] names[i];
    delete[] names;

}

CROOT_API void R_MinuitLimit(double (*fun)(int np, double *xval), int npar, 
    double *par, double *step, double *min, double *max, 
    double *outpar, double *err,  double *chiq){

    int    i;

    // Assigning names...
        // Assigning names...
    char **names = new char*[npar];
    for (i=0;i<npar;i++) {
      std::string str = GetStringName("p",i);
      names[i] = new char[str.length()+1];
      strncpy(names[i], str.c_str(), str.length());
    }

    R_Minuit(fun," ",npar,names,par,step,min,max,outpar,err,chiq);

    for (i=0; i<npar; i++) delete[] names[i];
    delete[] names;

}



//------------------------------------------------------------------
//
// ** Aesthetic **
//
// R_Line
// R_Text
// R_AxisTitles
// R_3DAxisTitles
// R_PrintPicture
//
//----------------------------------------------------------------

CROOT_API void R_Line(double x1, double y1, double x2, double y2){

  TLine l;
  l.SetBit(kCanDelete);  
  l.DrawLine(x1,y1,x2,y2);
  l.SetBit(kCanDelete);
  gPad->Modified();
  if (ROOT_lupdate==1) R_Update();;

}

CROOT_API void R_Text(double x, double y, const char *opt){

  TLatex t;
  t.SetBit(kCanDelete);
  t.DrawLatex(x,y,opt);
  t.SetBit(kCanDelete);
  gPad->Modified();
  if (ROOT_lupdate==1) R_Update();;

}


CROOT_API void R_AxisTitles(const char *optx,const char *opty){

  TH1F *hf           =  (TH1F *) gPad->GetPrimitive("hframe");
  TH1F *h1           =  (TH1F *) Find_ObjectInPad("TH1");
  TH2F *h2           =  (TH2F *) Find_ObjectInPad("TH2");
  TGraph *xgr        =  (TGraph *) Find_ObjectInPad("TGraph");
  TGraphErrs *xgre =  (TGraphErrs *) Find_ObjectInPad("TGraphErrors");

  if (hf != NULL){
    hf->SetXTitle(optx);
    hf->SetYTitle(opty);
  } else if (h1 != NULL){
    h1->SetXTitle(optx);
    h1->SetYTitle(opty); 
  } else if (h2 != NULL) {
    h2->SetXTitle(optx);
    h2->SetYTitle(opty);
  } else if (xgr  != NULL) {
    xgr->GetHistogram()->SetXTitle(optx);
    xgr->GetHistogram()->SetYTitle(opty);
  } else if (xgre != NULL) {
    xgre->GetHistogram()->SetXTitle(optx);
    xgre->GetHistogram()->SetYTitle(opty);
  }

  if (ROOT_loffx != 0) 
    R_HandleTitleOffset(ROOT_loffx,"X");
  else if (ROOT_loffy != 0) 
    R_HandleTitleOffset(ROOT_loffy,"Y");

  gPad->Modified();
  if (ROOT_lupdate==1) R_Update();

}

CROOT_API void R_3DAxisTitles(const char *optx,const char *opty, const char *optz){


  TAxis3D *ax =  (TAxis3D *) gPad->FindObject("axis3druler");

  if (ax != NULL){
    ax->SetXTitle(optx);
    ax->SetYTitle(opty);
    ax->SetZTitle(optz);
//    ax->Paint(); 
    if (ROOT_loffx != 0) 
      R_HandleTitleOffset(ROOT_loffx,"X");
    else if (ROOT_loffy != 0) 
      R_HandleTitleOffset(ROOT_loffy,"Y");
    else if (ROOT_loffz != 0) 
      R_HandleTitleOffset(ROOT_loffz,"Z");
  } else {
    std::cout << "%CROOT error: no 3D frame drawn" << std::endl;
  } 

  gPad->Modified();
  if (ROOT_lupdate==1) R_Update();;

}

CROOT_API void R_PrintPicture(const char *opt){
  ROOT_canv->Print(opt);
}
 








