// The following ifdef block is the standard way of creating macros which make exporting 
// from a DLL simpler. All files within this DLL are compiled with the CCERN_EXPORTS
// symbol defined on the command line. this symbol should not be defined on any project
// that uses this DLL. This way any other project whose source files include this file see 
// CCERN_API functions as being imported from a DLL, wheras this DLL sees symbols
// defined with this macro as being exported.

#ifdef CROOT_DLL_EXPORTS
#define CROOT_API extern "C" __declspec(dllexport) 
#else
#ifdef __cplusplus
#define CROOT_API extern "C"
#else
#define CROOT_API
#endif
#endif


// Initialization, pad handling

CROOT_API void R_Init(void);
CROOT_API void R_End(void);
CROOT_API void R_Divide(int nx, int ny);
CROOT_API void R_cd(int nn);
CROOT_API void R_Update(void);
CROOT_API void R_Delete(const char *name);
CROOT_API void R_Interact(void);


// Basics graphical routines for 2 and 3D

CROOT_API void R_DrawFrame(double xmin, double xmax, double ymin, double ymax);
CROOT_API void R_ResizeFrame(double xmin, double xmax, double ymin, double ymax);
CROOT_API void R_Graph(int n, double *x, double *y, const char *opt);
CROOT_API void R_GraphObj(const char *name, int n, double *x, double *y, const char *opt);
CROOT_API void R_GraphErrors(int n, double *x, double *y, double *ex, double *ey, const char *opt);
CROOT_API void R_GraphErrorsObj(const char *name, int n, double *x, double *y, double *ex, double *ey, const char *opt);
CROOT_API void R_SetAxis(const char *name, double xmin, double xmax, double ymin, double ymax);
CROOT_API void R_PlotFunc1D(double fun(double), double xmin, double xmax, const char *opt);
CROOT_API void R_PlotFuncPar1D(double fun(double, double *), int npar, double *par, double xmin, double xmax, const char *opt);
CROOT_API void R_Draw3DFrame(double xmin, double xmax, double ymin, double ymax, double zmin, double zmax, const char *opt);
CROOT_API void R_3DGraph(int n, double *x, double *y, double *z, const char *opt);
CROOT_API void R_3DGraphObj(const char *name, int n, double *x, double *y, double *z, const char *opt);

// Histogram
CROOT_API void R_CreateHist1D(const char *hist, const char *test, int ncx, float lowx, float upx);
CROOT_API void R_FillHist1D(const char *hist, float x, float w);
CROOT_API void R_CreateHist2D(const char *hist, const char *test, int ncx, float lowx, float upx, int ncy, float lowy, float upy);
CROOT_API void R_FillHist2D(const char *hist, float x, float y, float w);
CROOT_API void R_PlotHist(const char *hist, const char *opt);
CROOT_API void R_ResetHist(const char *hist);
CROOT_API void R_Modify(const char *name);
CROOT_API double R_GetBinContentHist1D(const char *hist, int bin);
CROOT_API double R_GetBinContentHist2D(const char *hist, int binx, int biny);
CROOT_API double R_GetEntries(const char *hist);
CROOT_API int    R_GetBinNumber(const char *hist, double x, const char *axis);
CROOT_API double R_GetBinCenter(const char *hist, int ibin, const char *axis);
CROOT_API int    R_GetNBins(const char *hist, const char *axis);
CROOT_API void   R_SetRangeHist(const char *hist, int imin, int imax, const char *axis);
CROOT_API double R_GetRMS(const char *hist, int axis);
CROOT_API double R_GetMean(const char *hist, int axis);
CROOT_API double R_GetIntegral(const char *hist, const char *opt);

// Minuit
CROOT_API void R_Minuit(double (*fun)(int np, double *xval),  const char *mode, int npar, 
                char **cpar, double *par, double *step, double *min, double *max, 
	        double *outpar, double *err,  double *chiq);
CROOT_API void R_MinuitLimit(double (*fun)(int np, double *xval), int npar, 
                double *par, double *step, double *min, double *max, 
	        double *outpar, double *err,  double *chiq);
CROOT_API void R_MinuitSimpl(double (*fun)(int np, double *xval), int npar, double *par, 
   	      double *outpar, double *err,  double *chiq);

// Aesthetic
CROOT_API void R_Text(double x, double y, const char *opt);
CROOT_API void R_Line(double x1, double y1, double x2, double y2);
CROOT_API void R_AxisTitles(const char *cx, const char *cy);
CROOT_API void R_3DAxisTitles(const char *cx, const char *cy,const char *cz);
CROOT_API void R_PrintPicture(const char *opt);

// Settings
CROOT_API void R_SetOptLogx(short int iopt);
CROOT_API void R_SetOptLogy(short int iopt);
CROOT_API void R_SetOptLogz(short int iopt);
CROOT_API void R_SetOptStat(int iopt);
CROOT_API void R_SetGrid(short int iopt);
CROOT_API void R_SetWait(short int iopt);
CROOT_API void R_SetUpdate(short int iopt);
CROOT_API void R_SetMarkerStyle(short int iopt);
CROOT_API void R_SetMarkerColor(short int iopt);
CROOT_API void R_SetMarkerSize(float fopt);
CROOT_API void R_SetLineStyle(short int iopt);
CROOT_API void R_SetLineColor(short int iopt);
CROOT_API void R_SetLineWidth(short int iopt);
CROOT_API void R_SetHistLineStyle(short int iopt);
CROOT_API void R_SetHistLineColor(short int iopt);
CROOT_API void R_SetHistFillStyle(short int iopt);
CROOT_API void R_SetHistFillColor(short int iopt);
CROOT_API void R_SetHistLineWidth(short int iopt);
CROOT_API void R_SetFuncStyle(short int iopt);
CROOT_API void R_SetFuncColor(short int iopt);
CROOT_API void R_SetFuncWidth(short int iopt);
CROOT_API void R_SetFillColor(short int iopt);
CROOT_API void R_SetTitleOffset(float off, const char *opt);

